@@ -54,6 +54,7 @@ static const boost::uint32_t B100_CTRL_MSG_SID = 20;
 static const double          B100_DEFAULT_TICK_RATE = 64e6;
 static const size_t          B100_MAX_PKT_BYTE_LIMIT = 2048;
 static const std::string     B100_EEPROM_MAP_KEY = "B100";
+static const size_t          B100_MAX_RATE_USB2  =  32000000; // bytes/s
 
 #define I2C_ADDR_TX_A       (I2C_DEV_EEPROM | 0x4)
 #define I2C_ADDR_RX_A       (I2C_DEV_EEPROM | 0x5)
@@ -105,8 +106,6 @@ public:
     bool recv_async_msg(uhd::async_metadata_t &, double);
 
 private:
-    uhd::property_tree::sptr _tree;
-
     //controllers
     fifo_ctrl_excelsior::sptr _fifo_ctrl;
     i2c_core_200::sptr _fpga_i2c_ctrl;
@@ -129,11 +128,6 @@ private:
     uhd::usrp::dboard_manager::sptr _dboard_manager;
     uhd::usrp::dboard_iface::sptr _dboard_iface;
 
-    //device properties interface
-    uhd::property_tree::sptr get_tree(void) const{
-        return _tree;
-    }
-
     std::vector<boost::weak_ptr<uhd::rx_streamer> > _rx_streamers;
     std::vector<boost::weak_ptr<uhd::tx_streamer> > _tx_streamers;
 
